/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 1991-2005 OpenCFD Ltd.
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM; if not, write to the Free Software Foundation,
    Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

Application
    setProfileRANS

Description

\*---------------------------------------------------------------------------*/

#include "fvCFD.H"
#include "incompressible/singlePhaseTransportModel/singlePhaseTransportModel.H"
#if WM_PROJECT_VERSION_MAJOR == 1 && WM_PROJECT_VERSION_MINOR < 5
#include "incompressible/turbulenceModel/turbulenceModel.H"
#else
#include "RAS/incompressible/RASModel/RASModel.H"
typedef incompressible::RASModel turbulenceModel;
#endif
#include "interpolateXY.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

int main(int argc, char *argv[])
{

#   include "setRootCase.H"

#   include "createTime.H"
#   include "createMesh.H"
#   include "createFields.H"

#   include "createPhi.H"

    singlePhaseTransportModel laminarTransport(U, phi);

    autoPtr<turbulenceModel> turbulence
    (
        turbulenceModel::New(U, phi, laminarTransport)
    );

    IOdictionary setProfileRANSDict
        (
            IOobject
            (
                "setProfileRANSDict",
                runTime.system(),
                mesh,
                IOobject::MUST_READ,
                IOobject::NO_WRITE
            )
        );

    const wordList patchNames = setProfileRANSDict.lookup("patchNames");
    const scalar theta(readScalar(setProfileRANSDict.lookup("theta")));

    Info << "\nRotation angle = " << theta << " [degree]" << endl;

    const scalar thetaCos=std::cos(mathematicalConstant::pi*theta/180.0);
    const scalar thetaSin=std::sin(mathematicalConstant::pi*theta/180.0);

    //
    // Read profile data from dict file
    // 
    typedef VectorSpace<Vector<scalar>, scalar, 4> quad;
    typedef VectorSpace<Vector<scalar>, scalar, 2> pair;

    // U
    List<quad> UProfile(setProfileRANSDict.lookup("UProfile"));
    scalarField hOldU(UProfile.size());
    scalarField inletUOldX(UProfile.size());
    scalarField inletUOldY(UProfile.size());
    scalarField inletUOldZ(UProfile.size());
    forAll(UProfile, i)
    {
        hOldU[i] = UProfile[i][0];
        inletUOldX[i] = UProfile[i][1];
        inletUOldY[i] = UProfile[i][2];
        inletUOldZ[i] = UProfile[i][3];
    }

    // k
    List<pair> kProfile(setProfileRANSDict.lookup("kProfile"));
    scalarField hOldK(kProfile.size());
    scalarField inletKOld(kProfile.size());
    forAll(kProfile, i)
    {
        hOldK[i] = kProfile[i][0];
        inletKOld[i] = kProfile[i][1];
    }

    // epsilon
    List<pair> epsilonProfile(setProfileRANSDict.lookup("epsilonProfile"));
    scalarField hOldEpsilon(epsilonProfile.size());
    scalarField inletEpsilonOld(epsilonProfile.size());
    forAll(epsilonProfile, i)
    {
        hOldEpsilon[i] = epsilonProfile[i][0];
        inletEpsilonOld[i] = epsilonProfile[i][1];
    }

    forAll(patchNames, patchNameI)
    {    
        // get index of patch
        label inletPatchID = mesh.boundaryMesh().findPatchID(patchNames[patchNameI]);

        vectorField& inletU = U.boundaryField()[inletPatchID]; 
        scalarField& inletK = turbulence->k()().boundaryField()[inletPatchID]; 
        scalarField& inletEpsilon = turbulence->epsilon()().boundaryField()[inletPatchID]; 
        // get coordinate for face centre of inlet boundary 
        volVectorField centres = U.mesh().C();
        vectorField& centresb = centres.boundaryField()[inletPatchID];
        scalarField hNew = centresb.component(vector::Z);

        // interpolate and rewrite U, k, epsilon
        scalarField inletUNewX = interpolateXY(hNew, hOldU, inletUOldX);
        scalarField inletUNewY = interpolateXY(hNew, hOldU, inletUOldY);
        scalarField inletUNewZ = interpolateXY(hNew, hOldU, inletUOldZ);
        scalarField inletURotateX=inletUNewX*thetaCos - inletUNewY*thetaSin;
        scalarField inletURotateY=inletUNewX*thetaSin + inletUNewY*thetaCos;
        inletU
            = inletURotateX*vector(1,0,0)
            + inletURotateY*vector(0,1,0)
            + inletUNewZ*vector(0,0,1);
        inletK= interpolateXY(hNew, hOldK, inletKOld);
        inletEpsilon =  interpolateXY(hNew, hOldEpsilon, inletEpsilonOld);

        Info << "\nRewrite " << patchNames[patchNameI] << " boundary" << endl;
    } 

    U.write();
    turbulence->k()().write();
    turbulence->epsilon()().write();


    Info<< "end" << endl;
        
    return(0);
}

// ************************************************************************* //
